@@ -23,15 +23,17 @@ are cached in the local filesystem and can be used by the recording module.
 """
 import tempfile
 import os
-import gst
-
-from types import StringType
 
 # Local imports
 import urlutil    
 import operations
 import audio
 
+def silent_unlink(filename):
+    try:
+        os.unlink(filename)
+    except OSError:
+        pass
 
 class GetMusic(operations.MeasurableOperation, operations.OperationListener):
     """"
@@ -135,28 +137,20 @@ class GstSourceToWavListener(operations.OperationListener):
         if event.id == operations.SUCCESSFUL:
             self.__parent.cache[self.__parent.unique_music_id(self.__music)] = GstCacheEntry(self.__filename, True)
         else:
-            os.unlink(self.__filename)
+            silent_unlink(self.__filename)
         
 class GstMusicPool(MusicPool):
+    temporary_dir = None
+    
     def __init__(self):
         self.__cache = {}
-        self.__temp_dir = None
+
     ############################################################################
     # Properties
     
     # read-only
     cache = property(lambda self: self.__cache)
     
-    # temporaryDir
-    def setTemporaryDir(self, temp_dir):
-        assert temp_dir == None or isinstance(temp_dir, StringType),\
-               "Directory must be a string. Or None for default."
-        self.__temp_dir = temp_dir
-    
-    def getTemporaryDir(self):
-        return self.__temp_dir
-    
-    temporaryDir = property(getTemporaryDir, setTemporaryDir)
     ############################################################################
     
     def is_available(self, music):
@@ -177,7 +171,7 @@ class GstMusicPool(MusicPool):
         assert not self.is_available(music)
         source = self.get_source()
         
-        handle, filename = tempfile.mkstemp(suffix = ".wav", dir = self.temporaryDir)
+        handle, filename = tempfile.mkstemp(suffix=".wav", dir=self.temporary_dir)
         os.close(handle)
         
         our_listener = GstSourceToWavListener(self, filename, music)
@@ -190,12 +184,22 @@ class GstMusicPool(MusicPool):
         for key in self.cache:
             entry = self.cache[key]
             if entry.is_temp:
-                os.unlink(entry.filename)
+                silent_unlink(entry.filename)
         self.__cache = {}
         
     def __del__(self):
         self.clear()
     
+    def remove_music(self, music):
+        music_id = self.unique_music_id(music)
+        try:
+            entry = self.cache.pop(music_id)
+            if entry.is_temp:
+                silent_unlink(entry.filename)
+
+        except KeyError:
+            pass
+    
     def unique_music_id(self, music):
         pass
 
@@ -256,7 +260,6 @@ class FetchMusicListPriv(operations.OperationListener):
             self.parent._propagate(evt)
             return
             
-        assert isinstance(evt.source, GetMusic)
         if evt.id != operations.SUCCESSFUL:
             return
             
@@ -265,10 +268,8 @@ class FetchMusicListPriv(operations.OperationListener):
         filename = pool.get_filename(uri)
         m = self.get_music_from_uri(uri)
 
-        if m:
-            m["cache_location"] = filename
-        else:
-            assert False, "uri '%s' was not found in music list." %(uri)
+        m["cache_location"] = filename
+
         
     def before_operation_starts(self, event, operation):
         e = operations.Event(self.parent)
