@@ -38,6 +38,7 @@ import urlutil
 from gtkutil import DictStore
 from operations import OperationsQueue
 from gdkpiechart import SerpentineUsage
+from common import parse_key_event
 
 ################################################################################
 # Operations used on AudioMastering
@@ -377,7 +378,7 @@ class GtkMusicList (MusicList):
 # Audio Mastering widget
 #    
 
-class AudioMasteringMusicListener (MusicListListener):
+class AudioMasteringMusicListener(MusicListListener):
     def __init__ (self, audio_mastering):
         self.__master = audio_mastering
         
@@ -386,6 +387,10 @@ class AudioMasteringMusicListener (MusicListListener):
     
     def on_musics_removed (self, e, rows):
         self.__master.update_disc_usage()
+        
+        for row in rows:
+            self.__master.pool.remove_music(row['location'])
+            
 
 class HintsFilter (object):
     __priority = 0
@@ -540,6 +545,7 @@ class AudioMastering (gtk.VBox, operations.Listenable):
         ('text/plain', 0, 2),
         ('STRING', 0, 3),
     ]
+    
     def __init__ (self, application):
         gtk.VBox.__init__ (self)
         self._application = weakref.ref(application)
@@ -558,13 +564,23 @@ class AudioMastering (gtk.VBox, operations.Listenable):
         
         self.add (g.get_widget ("audio_container"))
         
-        self.__setup_track_list (g)
-        self.__setup_container_misc (g)
+        self.__setup_track_list(g)
+        self.__setup_container_misc(g)
+        self.connect("key-press-event", self.on_key_pressed)
+        self.keys = {
+            "Delete": self.remove_selected,
+        }
+    
+    def on_key_pressed(self, widget, evt):
+        try:
+            self.keys[parse_key_event(evt)]()
+        except KeyError:
+            pass
     
     def __set_disc_size (self, size):
         assert size in AudioMastering.disc_sizes
         self.__disc_size = size
-        self.__size_list.set_active (AudioMastering.disc_sizes.index(size))
+        self.__size_list.set_active(AudioMastering.disc_sizes.index(size))
         self.update_disc_usage()
         e = operations.Event (self)
         for l in self.listeners:
@@ -580,6 +596,8 @@ class AudioMastering (gtk.VBox, operations.Listenable):
     
     disc_size_widget = property (lambda self: self.__size_list)
     
+    pool = property(lambda self: self._application().cache_pool)
+    
     def __setup_container_misc (self, g):
         self.__size_list = g.get_widget ("size_list")
         self.__usage_label = g.get_widget ("usage_label")
@@ -626,7 +644,7 @@ class AudioMastering (gtk.VBox, operations.Listenable):
         
         # Listen for drag-n-drop events
         lst.set_reorderable (True)
-        #XXX pygtk bug here
+
         lst.enable_model_drag_source (gtk.gdk.BUTTON1_MASK,
                                       AudioMastering.DND_TARGETS,
                                       gtk.gdk.ACTION_DEFAULT |
@@ -637,6 +655,7 @@ class AudioMastering (gtk.VBox, operations.Listenable):
                                     gtk.gdk.ACTION_MOVE)
         lst.connect ("drag_data_received", self.__on_dnd_drop)
         lst.connect ("drag_data_get", self.__on_dnd_send)
+        
     
     def __generate_track (self, col, renderer, tree_model, treeiter, user_data = None):
         index = tree_model.get_path(treeiter)[0]
