@@ -32,6 +32,7 @@ import xspf
 import gtkutil
 import release
 import urlutil
+import tempfile
 
 from converting import GvfsMusicPool
 from common import SafeFileWrite
@@ -40,6 +41,20 @@ GCONF_DIR = "/apps/serpentine"
 RAT_GCONF_DIR = "/apps/rat"
 NCB_GCONF_DIR = "/apps/nautilus-cd-burner"
 
+# Get system's temporary dir
+TEMP_DIR = tempfile.gettempdir()
+HOME_DIR = path.expanduser("~")
+
+def _is_dir_writable(dirname):
+    # Try to open the local file
+    try:
+        is_ok = path.isdir(dirname) and os.access(dirname, os.W_OK)
+    except OSError, err:
+        is_ok = False
+            
+    return is_ok
+
+
 for gconf_dir in (GCONF_DIR, RAT_GCONF_DIR, NCB_GCONF_DIR):
     gconf.client_get_default ().add_dir (gconf_dir, gconf.CLIENT_PRELOAD_NONE)
 
@@ -122,16 +137,15 @@ class WriteSpeed:
         specify_speed.connect ("toggled", self.__on_specify_speed)
         g.get_widget ("refresh_speed").connect ("clicked", self.__on_refresh_speed)
         
-        # init specify speed box sensitivity
-        self.__on_specify_speed (specify_speed)
-        
         # No default value set, set it to 99
         if self.__speed.data == 0:
             self.__speed.data = 99
 
         self.__update_speed ()
         self.__speed.sync_widget()
-        self.__speed.widget.set_sensitive (specify_speed.get_active ())
+        
+        # init specify speed box sensitivity
+        self.__on_specify_speed (specify_speed)
         
     def get(self):
         assert self.get_drive() is not None
@@ -159,7 +173,9 @@ class WriteSpeed:
         self.__speed.data = val
         
     def __on_specify_speed (self, widget, *args):
-        self.__specify_speed.set_sensitive (widget.get_active ())
+        # Sometimes the spinbox would get insensitive, so make sure it's not
+        self.__speed.widget.set_sensitive(widget.get_active())
+        self.__specify_speed.set_sensitive(widget.get_active())
     
 
 class RecordingPreferences (object):
@@ -177,8 +193,8 @@ class RecordingPreferences (object):
         filename = locations.get_data_file("serpentine.glade")
         g = glade.XML (filename, "preferences_dialog")
         self.__dialog = g.get_widget ("preferences_dialog")
-        self.dialog.connect ("destroy-event", self.__on_destroy)
-        self.dialog.set_title ("")
+        self.dialog.connect("destroy-event", self.__on_destroy)
+        self.dialog.set_title(_("Serpentine Preferences"))
         
         # Drive selection
         drv = g.get_widget ("drive")
@@ -209,7 +225,7 @@ class RecordingPreferences (object):
         self.__tmp = gaw.GConfValue (
             key = ncb_temp_dir,
             data_spec = gaw.Spec.STRING,
-            default = "file:///tmp"
+            default = TEMP_DIR
         )
         
         # debug
@@ -219,6 +235,38 @@ class RecordingPreferences (object):
             default = False
         )
         
+        # size
+        self.__x = gaw.GConfValue(
+            key = GCONF_DIR + "/x",
+            data_spec = gaw.Spec.INT,
+            default = 0,
+        )
+        
+        self.__y = gaw.GConfValue(
+            key = GCONF_DIR + "/y",
+            data_spec = gaw.Spec.INT,
+            default = 0,
+        )
+
+        self.__width = gaw.GConfValue(
+            key = GCONF_DIR + "/width",
+            data_spec = gaw.Spec.INT,
+            default = 0,
+        )
+
+        self.__height = gaw.GConfValue(
+            key = GCONF_DIR + "/height",
+            data_spec = gaw.Spec.INT,
+            default = 0,
+        )
+        
+        # disc size
+        self.__disc_size = gaw.GConfValue(
+            key = GCONF_DIR + "/disc_size",
+            data_spec = gaw.Spec.INT,
+            default = 1,
+        )
+        
         # Pool
         self.__pool = GvfsMusicPool ()
         
@@ -273,15 +321,47 @@ class RecordingPreferences (object):
     useGnomeVfs = property(getUseGnomeVfs, setUseGnomeVfs)
     
     ###############
-    # temporaryDir
-    def getTemporaryDir (self):
-        tmp = self.__tmp.data
-        url = urlutil.UrlParse(tmp)
-        if url.is_local:
-            return url.path
-            
+    # temporary_dir
+    
+    def get_temporary_dir(self):
+        return self.__tmp.data
+    
+    def set_temporary_dir(self, value):
+        self.__tmp.data = value
         
-    temporaryDir = property (getTemporaryDir)
+    temporary_dir = property (get_temporary_dir, set_temporary_dir)
+    
+    def get_temporary_dirs(self):
+        # First try the GConf temporary dir, then try the system TEMP dir
+        # finally try on the HOME
+        temps = (self.temporary_dir, TEMP_DIR, HOME_DIR)
+        for temp in temps:
+            if urlutil.is_local(temp) and _is_dir_writable(temp):
+                yield urlutil.get_path(temp)
+    
+    def get_window_position(self):
+        return self.__x.data, self.__y.data
+    
+    def set_window_position(self, x_y):
+        self.__x.data, self.__y.data = x_y
+    
+    window_position = property(get_window_position, set_window_position)
+    
+    def get_window_size(self):
+        return self.__width.data, self.__height.data
+    
+    def set_window_size(self, width_height):
+        self.__width.data, self.__height.data = width_height
+    
+    window_size = property(get_window_size, set_window_size)
+    
+    def get_disc_size(self):
+        return self.__disc_size.data
+    
+    def set_disc_size(self, disc_size):
+        self.__disc_size.data = disc_size
+    
+    disc_size = property(get_disc_size, set_disc_size)
     
     ##############
     # useGap
@@ -322,22 +402,6 @@ class RecordingPreferences (object):
         return ret
         
     writeFlags = property (getWriteFlags)
-
-    ####################
-    # temporaryDirIsOk
-    def getTemporaryDirIsOk (self):
-        tmp = self.temporaryDir
-        # Try to open the local file
-        try:
-            is_ok = path.isdir (tmp) and os.access (tmp, os.W_OK)
-        except OSError, err:
-            print err
-            is_ok = False
-        return is_ok
-        
-    temporaryDirIsOk = property (getTemporaryDirIsOk,
-                                    doc=("Tests if temporary directory exists "
-                                    "and has write permissions."))
     
     
     #############################
